Unit Misc;

Interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, ShellApi;



type EFileCopyException = class(ESTreamError)
     public
       ErrorCode: Integer;
       ErrorType: Integer; // (ERROR_READ, ERROR_WRITE, ERROR_UNKNOWN)
       constructor Create(const Msg: String; const ErrCode: Integer; const ErrType: Integer);
     end;

const ERROR_READ = 1;
      ERROR_WRITE = 2;
      ERROR_UNKNOWN = 3; //Niet te achterhalen of fout tijdens lezen of schrijven gebeurde

      {$IFNDEF FPC}
        {$IFDEF WIN32}
        DirectorySeparator = '\';
        {$ENDIF}
        {$IFDEF LINUX}
        DirectorySeparator = '/';
        {$ENDIF}
      {$ENDIF}




procedure CaptureScreenRect( ARect: TRect; ABitmap: TBitmap );
function DiskTypeStr(Drive: Char): ShortString;
function DiskInDrive(Drive: Char): Boolean;
function ExecuteFile(const FileName: string): Boolean;
function ExecuteFileEx(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): Boolean;
function ExecutePrintFile(const FileName: string): Boolean;
function IsASCIIFile(Fn: String): Boolean;
procedure Sleep(ms: Integer);
function BrowseForFolder(const Title: string): string;
function BrowseForFolderEx(const Title: string; const Flag: integer): string;
function FileCopy(const Source, Dest: String; const OverWriteReadOnlyFiles: Boolean): LongInt;
function FileCopyVerbose(const Source, Dest: String; const OverWriteReadOnlyFiles: Boolean; PBar: TProgressBar): LongInt;
function GetWindowsTempPath: String;


type TReplaceFlags = set of (rfReplaceAll, rfIgnoreCase);

Function StringReplace(const S, OldPattern, NewPattern: string;  Flags: TReplaceFlags): string;



Implementation

uses ShlObj;

procedure Sleep(ms: Integer);
var Start: LongInt;
begin
  Start := GetTickCount;
  repeat
    Application.ProcessMessages;
  until (GetTickCount > Start + ms) or (GetTickCount < Start)
end;

function IsASCIIFile(Fn: String): Boolean;
var Buf: array[0..255] of Byte;
    F: File;
    BR, i: Integer;
    OldFileMode: Integer;
const ControlChars = [1..31] - [10,13,26];
      //Alle ASCII tekens beneden spatie, m.u.v. #0, LineFeed, CarriageReturn, EOF
begin
  Result := False;
  OldFileMode := fileMode;
  FileMode := fmOpenRead or fmShareDenyNone;
  AssignFile(F,Fn);
  Try//Try..Finally
    Try//Try..except
      Reset(F,1);
      BlockRead(F,Buf,SizeOf(Buf),BR);
      CloseFile(F);
      if BR < 1 then Exit;
      for i := 0 to BR - 1 do
      begin
        if Buf[i] in ControlChars then
        begin
          //showmessage('Teken nummer '+IntToStr(i)+' -> $'+IntToHex(Buf[i],2));
          Exit;
        end;
      end;
      Result := true;
    Except
      //showmessage('InOutError !!');
      {$I-}CloseFile(F); InOutRes := 0; {$I+}
    end;
  Finally
    Filemode := OldFileMode;
  end;
end;



procedure CaptureScreenRect( ARect: TRect; ABitmap: TBitmap );
var
  ScreenDC: HDC;
begin
  with ABitmap, ARect do
  begin
    Width := Right - Left;
    Height := Bottom - Top;
    ScreenDC := GetDC( 0 );
    try
      BitBlt( Canvas.Handle, 0, 0, Width, Height, ScreenDC,
        Left, Top, SRCCOPY );
    finally
      ReleaseDC( 0, ScreenDC );
    end;
  end;
end;

(*
{ Use this to capture the entire screen... }
procedure CaptureScreen;
begin
  with Screen do
    CaptureScreenRect( Rect( 0, 0, Width, Height ), Form1.Image1.Picture.Bitmap);
end;
*)

function DiskTypeStr(Drive: Char): ShortString;
var
  i: word;
begin
  i := GetDriveType(PChar(LowerCase(Drive+':\')));
  case i of
    { Onbekend }
    0: result := 'onbekend type';
    { bestaat niet }
    1: result := 'disk bestaat niet';
    { Floppy }
    DRIVE_REMOVABLE: result := 'floppy';
    { Harde Schijf }
    DRIVE_FIXED: result := 'harddisk';
    { Net eenheid }
     DRIVE_REMOTE: result := 'netwerk disk';
    { CD-ROM }
    DRIVE_CDROM: result := 'cd-rom';
    { Ram Drive }
    DRIVE_RAMDISK: result := 'ram-disk';
  end;
end;

function DiskInDrive(Drive: Char): Boolean;
var
  ErrorMode: word;
begin
  { maak er een hoofdletter van }
  Drive := UpCase(Drive);
  if not (Drive in ['A'..'Z']) then
    raise EConvertError.Create('Geen geldige stationsaanduiding: '+Drive+':');
  { schakel kritieke fouten uit }
  ErrorMode := SetErrorMode(SEM_FailCriticalErrors);
  try
    { drive 1 = a, 2 = b, 3 = c, etc. }
    if DiskSize(Ord(Drive) - Ord('A') + 1) = -1 then
      Result := False
    else
      Result := True;
  finally
    { restore old error mode }
    SetErrorMode(ErrorMode);
  end;
end;


function ExecuteFileEx(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  AHandle := ShellExecute(Application.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
  if AHandle <= 32 then Result := False else Result := True;
end;


function ExecuteFile(const FileName: string): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  AHandle := ShellExecute(Application.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, ''),
    StrPCopy(zDir, ''), SW_SHOWNORMAL);
  if AHandle <= 32 then Result := False else Result := True;
end;


function ExecutePrintFile(const FileName: string): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  AHandle := ShellExecute(Application.Handle, PChar('Print'),
    StrPCopy(zFileName, FileName), StrPCopy(zParams, ''),
    StrPCopy(zDir, ''), SW_SHOWNORMAL);
  if AHandle <= 32 then Result := False else Result := True;
//ERROR_NO_ASSOCIATION 1155 No application is associated with the specified file for this operation.
end;


{ ******* From: http://delphi.about.com/od/windowsshellapi/l/aa122803a.htm **********}

function BrowseDialogCallBack
  (Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM):
  integer stdcall;
//Centreert de BrowseFolder dialoog op het scherm
var
  wa, rect : TRect;
  dialogPT : TPoint;
begin
  //center in work area
  if uMsg = BFFM_INITIALIZED then
  begin
    //wa := Screen.WorkAreaRect;
    GetWindowRect(Wnd, Rect);
(*
    dialogPT.X := ((wa.Right-wa.Left) div 2) -
                  ((rect.Right-rect.Left) div 2);
    dialogPT.Y := ((wa.Bottom-wa.Top) div 2) -
                  ((rect.Bottom-rect.Top) div 2);
*)
    dialogPT.X := ((Screen.Width) div 2) -
                  ((rect.Right-rect.Left) div 2);
    dialogPT.Y := ((Screen.Height) div 2) -
                  ((rect.Bottom-rect.Top) div 2);
    MoveWindow(Wnd,
               dialogPT.X,
               dialogPT.Y,
               Rect.Right - Rect.Left,
               Rect.Bottom - Rect.Top,
               True);
  end;

  Result := 0;
end; (*BrowseDialogCallBack*)

{ ******* From: http://delphi.about.com/od/windowsshellapi/l/aa122803a.htm **********}

function BrowseForFolderEx(const Title: string; const Flag: integer): string;
var
  lpItemID : PItemIDList;
  BrowseInfo : TBrowseInfo;
  DisplayName : array[0..MAX_PATH] of char;
  TempPath : array[0..MAX_PATH] of char;
begin
  Result:='';
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  with BrowseInfo do begin
    hwndOwner := Application.Handle;
    pszDisplayName := @DisplayName;
    lpszTitle := PChar(Title);
    ulFlags := Flag;
    lpfn := BrowseDialogCallBack;
  end;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  if lpItemId <> nil then begin
    SHGetPathFromIDList(lpItemID, TempPath);
    Result := TempPath;
    GlobalFreePtr(lpItemID);
  end;
end;


function BrowseForFolder(const Title: string): string;
//Hardcoded: BIF_RETURNONLYFSDIRS als Flag
var
  lpItemID : PItemIDList;
  BrowseInfo : TBrowseInfo;
  DisplayName : array[0..MAX_PATH] of char;
  TempPath : array[0..MAX_PATH] of char;
begin
  Result:='';
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  with BrowseInfo do begin
    hwndOwner := Application.Handle;
    pszDisplayName := @DisplayName;
    lpszTitle := PChar(Title);
    ulFlags := BIF_RETURNONLYFSDIRS;
    lpfn := BrowseDialogCallBack;
  end;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  if lpItemId <> nil then begin
    SHGetPathFromIDList(lpItemID, TempPath);
    Result := TempPath;
    GlobalFreePtr(lpItemID);
  end;
end;

constructor EFileCopyException.Create(const Msg: String; const ErrCode: Integer; const ErrType: Integer);
begin
  inherited Create(Msg);
  ErrorCode := ErrCode;
  ErrorType := ErrType;
end;


function FileCopy(const Source, Dest: String; const OverWriteReadOnlyFiles: Boolean): LongInt;
//Return nr. of bytes copied, 0 if exception occurs
begin
  Result := FileCopyVerbose(Source, Dest, OverWriteReadOnlyFiles, nil);
end;


function FileCopyVerbose(const Source, Dest: String; const OverWriteReadOnlyFiles: Boolean; PBar: TProgressBar): LongInt;
//Return nr. of bytes copied, 0 if exception occurs
var InStream: TFileStream;
    OutStream: TFileStream;
    buf: array[0..32767] of byte; {32 K}
    bRead, bWrit: LongInt;
    FAttr: Integer; //File Attribute
    Tm: Integer; //File Timestamp
    FileSize: LongInt;
    PBarRange: LongInt;
    SavedMode: uINT;
    //Extra controle op lees/schrijffouten
    TotalBytesRead, BytesCopied: LongInt; //Max 2 Gig on 32 bit compiler...
procedure EraseDest;
var F: File;
begin
  Try
    OutStream.Free;
    OutStream := nil;
    if FileExists(Dest) then
    begin
      AssignFile(F, Dest);
      Erase(F);
      CloseFile(F);
    end;
  Except
    //Negeren
  end;
end;
begin
  BytesCopied := 0;
  TotalBytesRead := 0;
  SavedMode := SetErrorMode(SEM_FAILCRITICALERRORS or SEM_NOOPENFILEERRORBOX);
  InStream := nil; //Anders kans op GPF's in finally gedeelte
  OutStream := nil;
  try//try..finally
    //Source openen
    try//try..except
      InStream := TFileStream.Create(Source, fmOpenRead or fmShareDenyWrite);
      Tm := FileGetDate(InStream.Handle); //Timestamp van Source; -1 = ERROR
      FAttr := FileGetAttr(Dest); //Negeer fouten bij lezen attribuut
    except on EStreamError do Raise EFileCopyException.Create('Fout bij lezen',GetLastError,ERROR_READ);
    end;//try..except
    //Dest aanmaken
    try//try..except
      if OverWriteReadOnlyFiles and FileExists(Dest) then
      begin
        if (FAttr > 0) and ((FAttr and faReadOnly) > 0) then FileSetAttr(Dest, faArchive);
        //als dat misgaat dan merkenwe dat in de volgende regel vanzelf ...
      end;
      OutStream := TFileStream.Create(Dest, fmCreate or fmShareDenyWrite);
    except on EStreamError do Raise EFileCopyException.Create('Fout bij schrijven',GetLastError,ERROR_WRITE);
    end;//try..except
    FileSize := InStream.Size;
    if (PBar <> nil) then
    begin
      PBar.Position := 0;
      PBarRange := PBar.Max - PBar.Min;
    end;
    //Begin kopieer actie
    repeat
      bRead := InStream.Read(buf, SizeOf(Buf));
      TotalBytesRead := TotalBytesRead + bRead;
      //Minder bytes gelezen dan buffergrootte, maar geen EOF
      if (bRead <> SizeOf(Buf)) and (TotalBytesRead <> FileSize) then
      begin
        // Dest moet verwijderd worden na mislukt kopieeractie!!
        EraseDest;
        Raise EFileCopyException.Create('Leesfout',GetLastError,ERROR_READ);
      end;
      bWrit := OutStream.Write(buf, bRead);
      //Dit is natuurlijk altijd fout
      if (bWrit <> bRead) then
      begin
        // Dest moet verwijderd worden na mislukt kopieeractie!!
        EraseDest;
        Raise EFileCopyException.Create('Schrijffout',GetLastError,ERROR_WRITE);
      end;
      BytesCopied := BytesCopied + bWrit;
      if (PBar <> nil) then
      begin
        if FileSize > 0 then PBar.Position := Round(PBarRange * (OutStream.Position / FileSize)) + PBar.Min
        else PBar.Position := PBar.Max;
        Application.ProcessMessages;
      end;
    until (bRead <> SizeOf(Buf)) or (bWrit <> bRead);
    if Tm <> -1 then FileSetDate(OutStream.Handle, Tm); //Negeer fouten bij zetten van tijd
    FileSetAttr(Dest, FAttr);//Negeer fouten bij zetten attribuut
  finally
    Result := BytesCopied;
    SetErrorMode(SavedMode);
    if InStream <> nil then InStream.Free;
    if OutStream <> nil then OutStream.Free;
  end;
end;


Function StringReplace(const S, OldPattern, NewPattern: string;  Flags: TReplaceFlags): string;
var
  Srch,OldP,RemS: string; // Srch and Oldp can contain uppercase versions of S,OldPattern
  P : Integer;
begin
  Srch:=S;
  OldP:=OldPattern;
  if rfIgnoreCase in Flags then
    begin
    Srch:=AnsiUpperCase(Srch);
    OldP:=AnsiUpperCase(OldP);
    end;
  RemS:=S;
  Result:='';
  while (Length(Srch)<>0) do
    begin
    P:=AnsiPos(OldP, Srch);
    if P=0 then
      begin
      Result:=Result+RemS;
      Srch:='';
      end
    else
      begin
      Result:=Result+Copy(RemS,1,P-1)+NewPattern;
      P:=P+Length(OldP);
      RemS:=Copy(RemS,P,Length(RemS)-P+1);
      if not (rfReplaceAll in Flags) then
        begin
        Result:=Result+RemS;
        Srch:='';
        end
      else
         Srch:=Copy(Srch,P,Length(Srch)-P+1);
      end;
    end;
end;

function GetWindowsTempPath: String;
var
   lng: DWORD;
   thePath: string;
begin
  SetLength(thePath, MAX_PATH) ;
  lng := GetTempPath(MAX_PATH, PChar(thePath)) ;
  SetLength(thePath, lng) ;
  if thePath[Length(thePath)] <> '\' then thePath := thePath + ']';
  Result := thePath;
end;


end.