unit u_dccopy;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  NlDlg32, FileCtrl, ComCtrls, ExtCtrls, StdCtrls, Buttons, Misc;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    FilterComboBox: TFilterComboBox;
    StartBtn: TBitBtn;
    CopyPanel: TPanel;
    ProgressBar: TProgressBar;
    FnLabel: TLabel;
    FileListBox: TFileListBox;
    OnlyRecentCheckBox: TCheckBox;
    ResolveCheckBox: TCheckBox;
    DatePrefixCheckBox: TCheckBox;
    CancelBtn: TBitBtn;
    SourceEdit: TEdit;
    DestEdit: TEdit;
    SourceBtn: TBitBtn;
    DestBtn: TBitBtn;
    function IsMoreRecent(const SDate: Integer; Dest: String): Boolean;
//    function FileCopy(const Source,Dest: String; const Attr, Tm: Integer): Boolean;
    procedure WalkPath(const SP: String);
    procedure StartBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    function GetAlternativeFileName(Fn: String): String;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OnlyRecentCheckBoxClick(Sender: TObject);
    procedure ResolveCheckBoxClick(Sender: TObject);
    procedure DatePrefixCheckBoxClick(Sender: TObject);
    procedure EnableControls;
    procedure DisableControls;
    procedure CancelBtnClick(Sender: TObject);
    procedure SrcBtnClick(Sender: TObject);
    procedure DestBtnClick(Sender: TObject);
    procedure SourceEditChange(Sender: TObject);
  private
    { Private declarations }
    SourceRoot, DestRoot: String;
    ErrorFlag, CancelFlag: Boolean;
    SystemError: LongInt;
    UsedatePrefix: Boolean;
    OnlyRecent: Boolean;
    ResolveConflict: Boolean;
    DestFileName: String;
    IniName: String;
    FileCounter: LongInt;
    TotalBytesCopied: Extended;
    UseLog: Boolean;
    LogFile: TextFile;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses ShellApi, Errors, IniFiles;
{$R *.DFM}


const DefaultFilter = 'Foto''s en films|*.jpg;*.jpeg;*.avi;*.mov;*.raw;*.tif;*.tiff|Alle bestanden|*.*';
      msgCopyOk = 'Klaar met kopiren.'^m^m'Totaal aantal bestanden: %d.'^m'Totaal aantal bytes: %s.';
      msgCopyCancel = 'Kopiren afgebroken door gebruiker.';
      msgCopyError = 'Er is een fout opgetreden tijdens het %s:'^m+
                             ^m'%s.'^m^m'"%s."';
      msgSameDir = 'Bronmap en doelmap zijn hetzelfde !';
      msgBusy = 'Bezig met kopiren . . .';
      msgPrefError = 'De opgegeven %smap is onjuist:'^m'%s';
      msgCreatDirError = 'Fout bij maken van map: %s.';
      msgCopyErrorUnknown = 'Er is een fout opegtreden tijdens het kopiren:'^m^m'Bron: %s'^m'Doel: %s'^m+
                               ^m'%s.'^m^m'"%s."';
      msgDirNotFound = '%s map niet gevonden:'^m'%s.';
      msgErrorSaveSettings = 'Fout bij opslaan van de instellingen';
      Kilo = 1024; Mega = Kilo*Kilo; Giga = Mega * Kilo;

function AddZero(Nr: Integer; Width: Integer): ShortString;
var S: ShortString;
begin
  S := IntToStr(Nr);
  While Length(S) < Width do S := '0'+ S;
  Result := S;
end;


procedure TForm1.EnableControls;
begin
  CopyPanel.Visible := False;
  SourceEdit.Enabled := True;
  DestEdit.Enabled := True;
  FilterComboBox.Enabled := True;
  OnlyRecentCheckBox.Enabled := True;
  ResolveCheckBox.Enabled := True;
  DatePrefixCheckBox.Enabled := True;
  StartBtn.Enabled := True;
end;


procedure TForm1.DisableControls;
begin
  CopyPanel.Visible := True;
  SourceEdit.Enabled := False;
  DestEdit.Enabled := False;
  FilterComboBox.Enabled := False;
  OnlyRecentCheckBox.Enabled := False;
  ResolveCheckBox.Enabled := False;
  DatePrefixCheckBox.Enabled := False;
  StartBtn.Enabled := False;
end;



function TForm1.GetAlternativeFileName(Fn: String): String;
var P,E,S: String;
    i: integer;
begin
  P := ExtractFilePath(Fn);
  Fn := ExtractFileName(Fn);
  E := ExtractFileExt(Fn);
  Fn := Copy(Fn,1,Length(Fn)-Length(E));
  for i := 1 to 99 do
  begin
    S := P + Fn + '_' + AddZero(i,2) + E;
    if not FileExists(S) then Break;
  end;
  if FileExists(S) then S := 'ERROR';
  Result := S;
end;



procedure TForm1.WalkPath(const SP: String);
//SP: SourcePath
const faAttr = faDirectory;
var SR: TSearchRec;
    i, FDate, HD, DosError: Integer;
    FDateTime: TDateTime;
    D,M,Y: Word;
    DateStr: String[11];
    Sn: ShortString;
    ErrMsg: String;
begin
  Application.ProcessMessages;

  //Kopieer eerst alles in deze map
  With FileListBox do
  begin//Alle in deze map bestanden die voldoen aan het filter kopieren
    Directory := SP;
    if (Items.Count > 0) then for i := 0 to Items.Count-1 do
    begin
      if ErrorFlag then Break;
      HD := FileOpen(Items[i],fmOpenRead or fmShareDenyNone);
      FDate := FileGetDate(HD);
      FileClose(HD);
      if (HD <> -1) then
      begin//Bestand bestaat echt ...
        if UseDatePrefix then
        begin
          FDateTime := FileDateToDateTime(FDate);
          Decodedate(FDateTime,Y,M,D);
          DateStr := AddZero(Y,4)+'_'+AddZero(M,2)+'_'+AddZero(D,2);
          if not DirectoryExists(DestRoot+DateStr) then
          begin
            try
              MkDir(DestRoot+DateStr);
            except
              on E: EInOutError do
              begin
                ErrorFlag := true;
                MessageDlg(Format(msgCreatDirError,[DestRoot+DateStr]),mtError,[mbOk],0);
                Exit;
              end;
            end;
          end;
          DateStr := DateStr + '\';
        end
        else DateStr := '';
        DestFileName := DestRoot+DateStr+items[i];
        if (not FileExists(DestFileName)) or
           (FileExists(DestFileName) and ((not OnlyRecent) or IsMoreRecent(FDate,DestFileName))) then
        begin//Ja, dit bestand moet gekopieerd worden
          FnLabel.Caption := Items[i];
          if FileExists(DestFileName) and ResolveConflict then
          begin
            DestFileName := GetAlternativeFilename(DestFileName);
          end;
          Application.ProcessMessages;
          if Directory[Length(Directory)] = '\' then Sn := Directory + Items[i]
          else Sn := Directory + '\' + Items[i];
          Try//try..except
            TotalBytesCopied := TotalBytesCopied + FileCopyVerbose(Sn, DestFileName,True,ProgressBar);
            FileCounter := FileCounter + 1;
          Except
            on E: EFileCopyException do
            begin
              ErrorFlag := True;
              SystemError := E.ErrorCode;
              case E.ErrorType of
                ERROR_READ: ErrMSg := Format(msgCopyError,['lezen van',Sn,NlErrorMsg(SystemError)]);
                ERROR_WRITE:ErrMSg := Format(msgCopyError,['schrijven naar',DestFileName,NlErrorMsg(SystemError)]);
                Else ErrMsg := Format(msgCopyErrorUnknown,[Sn,DestFileName,NlErrorMsg(SystemError)]);
              end;//case
              MessageDlg(ErrMsg,mtError,[mbOk],0);
            end;
            on E: EStreamError do
            begin
              ErrorFlag := True;
              SystemError := GetLastError;
              ErrMsg := Format(msgCopyErrorUnknown,[Sn,DestFileName,NlErrorMsg(SystemError)]);
              MessageDlg(ErrMsg,mtError,[mbOk],0);
            end;
          end;//try..except
          if UseLog then
          begin
            Try
              write(LogFile,Sn + ' --> ' + DestFileName + '  ');
              if ErrorFlag then writeln(LogFile, '[ERROR !]')
              else writeln(LogFile,'[Ok.]');
            Except on EInOutError do
              begin
                 UseLog := False;
                {$I-}CloseFile(LogFile); InOutRes := 0; {$I+}//Probeer bestand nog te sluiten ...
              end;
            end;//try..except
          end;
(*
          if ErrorFlag then
          begin//fout bij kopieren
            MessageDlg(Format(msgCopyExtendedErrorEx,
                       [Items[i],DestRoot+DateStr+Items[i],NlErrorMsg(SystemError)]),
                       mtError,[mbOk],0);
          end
          else FileCounter := FileCounter + 1;
*)
        end;
      end;//HD <> -1
    end;//for
  end;//with FileListBox do

  //Ga op zoek naar onderliggende mappen en roep deze functie z.n. recursief aan
  DosError := FindFirst(SP+'*.*',faAttr,SR);
  While (DosError = 0) and (not ErrorFlag) and not CancelFlag do
  begin
    Application.ProcessMessages;
    //Zorg voor screen update en afhandelen muiskliks op Afbreek-knop
    if (SR.Name[1] <> '.') then
    begin
      if ((SR.Attr and faDirectory) = faDirectory) then
      begin//Recurse directory
        Application.ProcessMessages;
        WalkPath(SP+SR.Name+'\');
        Application.ProcessMessages;
      end;
    end;//SR.Name[1] <> '.'
    DosError := FindNext(SR);
  end;//while Doserror = 0
  FindClose(SR);
end;





function TForm1.IsMoreRecent(const SDate: Integer; Dest: String): Boolean;
var HD: Integer;
    DDate: Integer;
begin
  Result := True;
  begin
    if not FileExists(Dest) then Exit;
    HD := FileOpen(Dest,fmOpenRead or fmShareDenyNone);
    DDate := FileGetDate(HD);
    FileClose(HD);
    if (HD = -1) then Exit;//-1 = Error: toch maar kopieren dan
    Result := (SDate > DDate);
  end;
end;



procedure TForm1.StartBtnClick(Sender: TObject);
var BytesString: String;
begin
  SourceRoot := SourceEdit.Text;
  if not DirectoryExists(SourceRoot) then
  begin
    MessageDlg(Format(msgDirNotFound,['Bron',SourceRoot]),mtError,[mbOk],0);
    exit;
  end;
  DestRoot := DestEdit.Text;
  if not DirectoryExists(DestRoot) then
  begin
    MessageDlg(Format(msgDirNotFound,['Doel',DestRoot]),mtError,[mbOk],0);
    exit;
  end;
  If CompareText(SourceRoot,DestRoot) = 0 then
  begin
    MessageDlg(msgSameDir,mtError,[mbOk],0);
    Exit;
  end;
  ErrorFlag := False;
  CancelFlag := False;
  FileCounter := 0;
  TotalBytesCopied := 0.0;
  if UseLog then
  begin
    Try
      writeln(LogFile,ParamStr(0));
      writeln(LogFile,'Kopieerproces gestart op: ' + FormatDateTime('dd-mm-yyyy hh:nn:ss',Now));
    Except on EInOutError do
      begin
        UseLog := False;
        {$I-}CloseFile(LogFile); InOutRes := 0; {$I+}//Probeer bestand nog te sluiten ...
      end;
    end;//try..except
  end;
  Try
    Screen.Cursor := crAppStart;//crHourGlass;
    DisableControls;
    WalkPath(SourceRoot);
  Finally
    Screen.Cursor := crDefault;
    EnableControls;
  end;
  if UseLog then
  begin
    Try
      writeln(LogFile,'Kopieerproces beindigd op: ' + FormatDateTime('dd-mm-yyyy hh:nn:ss',Now));
      writeln(LogFile);
    Except on EInOutError do
      begin
        UseLog := False;
        {$I-}CloseFile(LogFile); InOutRes := 0; {$I+}//Probeer bestand nog te sluiten ...
      end;
    end;//try..except
  end;
  if CancelFlag then MessageDlg(msgCopyCancel,mtInformation,[mbOk],0);
  if not (Cancelflag or ErrorFlag) then
  begin
    if TotalBytesCopied < Kilo then BytesString := Format('%.0f bytes',[TotalBytesCopied])
    else if TotalBytesCopied < Mega then BytesString := Format('%.1f Kb',[TotalBytesCopied/Kilo])
    else if TotalBytesCopied < Giga then BytesString := Format('%.1f Mb',[TotalBytesCopied/Mega])
    else if TotalBytesCopied >= Mega then BytesString := Format('%.1f Gb',[TotalBytesCopied/Giga]);
    MessageDlg(Format(msgCopyOk,[FileCounter,BytesString]),mtInformation,[mbOk],0);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var Ini: TIniFile;
    S: String;
    T, L: Integer;
begin

  FilterComboBox.FileList := FileListBox;
  FilterComboBox.Filter := DefaultFilter;

  UseDatePrefix := True;
  OnlyRecent := True;
  ResolveConflict := True;
  FileCounter := 0;
  FnLabel.Caption := '';

  IniName := ChangeFileExt(ParamStr(0),'.INI');
  Ini := TIniFile.Create(IniName);
  Try
    //Laatst gebruikte pad voor opslaan en openen inlezen
    S := Ini.ReadString('DCCOPY','Filter',DefaultFilter);
    if Pos('*.',S) > 0 then FilterComboBox.Filter := S;
    SourceEdit.Text := Ini.ReadString('DCCOPY','Bron',SourceEdit.Text);
    DestEdit.Text := Ini.ReadString('DCCOPY','Doel',DestEdit.Text);
    OnlyRecent := Ini.ReadBool('DCCOPY','AlleenRecenter',True);
    UseDatePrefix := Ini.ReadBool('DCCOPY','DatumPrefix',True);
    ResolveConflict := Ini.ReadBool('DCCOPY','ConflictVermijding',True);
    T := Ini.ReadInteger('DCCOPY','Top',-1);
    L := Ini.ReadInteger('DCCOPY','Left',-1);
    if (T >= 0) then
    begin
      While ((T + Height) > Screen.Height) do Dec(T);
    end;
    if (L >= 0) then
    begin
      While ((L + Width) > Screen.Width) do Dec(L);
    end;
    if (T >= 0) and (L >= 0) then Form1.SetBounds(L,T,Width,Height);
  Finally
    Ini.Free;
  End;

  DatePrefixCheckBox.Checked := UseDatePrefix;
  ResolveCheckBox.Checked := ResolveConflict;
  OnlyRecentCheckBox.Checked := OnlyRecent;

  UseLog := UpperCase(ParamStr(1)) = '/USELOG';
  if UseLog then
  begin
    Try
      AssignFile(LogFile,ChangeFileExt(ParamStr(0),'.log'));
      if FileExists(ChangeFileExt(ParamStr(0),'.log')) then Append(LogFile)
      else Rewrite(LogFile);
      ShortDateFormat := 'dd-mm-yyyy';

    Except on EInOutError do UseLog := False;
    end;
  end;

end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var Ini: TIniFile;
begin
  Try//try..except
    Ini := TIniFile.Create(IniName);
    Try
      //Laatst gebruikte pad voor opslaan en openen en ander instellingen bewaren in .INI bestand
      Ini.WriteString('DCCOPY','Filter',FilterComboBox.Filter);
      Ini.WriteString('DCCOPY','Bron',SourceEdit.Text);
      Ini.WriteString('DCCOPY','Doel',DestEdit.Text);
      Ini.WriteBool('DCCOPY','AlleenRecenter',OnlyRecent);
      Ini.WriteBool('DCCOPY','DatumPrefix',UseDatePrefix);
      Ini.WriteBool('DCCOPY','ConflictVermijding',ResolveConflict);
      Ini.WriteInteger('DCCOPY','Top',Top);
      Ini.WriteInteger('DCCOPY','Left',Left);
    Finally
      Ini.Free;
    End;
  Except
    on E: Exception do MessageDlg(msgErrorSaveSettings,mtError,[mbOk],0);
  end;

  if UseLog then
  begin
    Try
      CloseFile(LogFile);
    Except on EInOutError do ;
    end;
  end;
end;

procedure TForm1.OnlyRecentCheckBoxClick(Sender: TObject);
begin
  OnlyRecent := OnlyRecentCheckBox.Checked;
end;

procedure TForm1.ResolveCheckBoxClick(Sender: TObject);
begin
  ResolveConflict := ResolveCheckBox.Checked;
end;

procedure TForm1.DatePrefixCheckBoxClick(Sender: TObject);
begin
  UseDatePrefix := DatePrefixCheckBox.Checked;
end;

procedure TForm1.CancelBtnClick(Sender: TObject);
begin
  CancelFlag := True;
end;

procedure TForm1.SrcBtnClick(Sender: TObject);
var S: String;
begin
  S := BrowseForFolder('Selecteer bron map');
  if Length(S) = 0 then Exit;
  if S[Length(S)] <>  DirectorySeparator then S := S + DirectorySeparator;
  SourceEdit.Text := S;
end;

procedure TForm1.DestBtnClick(Sender: TObject);
var S: String;
begin
  S := BrowseForFolder('Selecteer doel map');
  if Length(S) = 0 then Exit;
  if S[Length(S)] <>  DirectorySeparator then S := S + DirectorySeparator;
  DestEdit.Text := S;
end;

procedure TForm1.SourceEditChange(Sender: TObject);
var S: String;
begin
  with (Sender as TEdit) do
  begin
    S := Text;
    if S[Length(S)] <>  DirectorySeparator then
    begin
      S := S + DirectorySeparator;
      Text := S;
    end;  
  end;
end;

end.
